; ---------------------------------------------------------------------------
; Minimal SFX player (YMZ example) (c) 2013 SyX
; ---------------------------------------------------------------------------
    LIST
    include "firmware.i"
    include "hardware.i"
    include "supergrafx.i"
    include "macros.i"

; ---------------------------------------------------------------------------
start
    ; Initialize Random Seed
    LD   A,R
    LD   (get_random_number.sm_seed + 1),A

    CALL print_menu

    CALL MC_WAIT_FLYBACK
    HALT
    
    ; Disable interrupts
    DI

    ; If you are going to use an external YMZ,
    ; you should set the frequency of the YMZs
    LD   BC,CTC_TIM0
    LD   HL,CTC_START_COUNTER * 256 + 1
    OUT  (C),H                      ; Set Timer 0 as clock for the YMZ
    OUT  (C),L                      ; 2 MHz clock for YMZ (CPC frequency)

    ; Initialize SFX player
    LD   HL,sfx_bank    ; SFX Bank address
    LD   A,1            ; 0 -> PSG | Rest -> YMZs
    CALL SFX_Init

    ; Wait until not key is pressed
    CALL no_key_press
main_loop
    CALL wait_vblank
    HALT
    HALT
    HALT
    
    CALL SFX_Play

    ; Read keys
    CALL KM_READ_KEY
    JR   NC,main_loop
    CALL get_random_number

    ; Add new SFX to be played
    LD   L,A                ; L = SFX number (0...255)
    LD   A,0                ; A = Channel number (0 = Free | 1 = A | 2 = B | 3 = C)
    CALL SFX_Send
    
    JR   main_loop

; ---------------------------------------------------------------------------
; Get an 7-bit random number
; RESULT:
;    A: New random number
;    Corrupts B.
; ---------------------------------------------------------------------------
get_random_number
.sm_seed
    LD   A,$00
    LD   B,A
    SLA  A
    XOR  B
    LD   B,A
    SRL  A
    XOR  B
    LD   B,A
    SLA  A
    SLA  A
    XOR  B
    LD   (.sm_seed + 1),A
    RES  7,A
    RET

; ---------------------------------------------------------------------------
; Wait vertical refresh using the hardware
wait_vblank
    LD   B,>PPI_B
.loop_wait_vblank
    IN   A,(C)
    RRA
    JR   NC,.loop_wait_vblank
    RET

; ---------------------------------------------------------------------------
; SFX player code
    include "sfx_player.asm"

; ---------------------------------------------------------------------------
; Wait until no key is pressed
; ---------------------------------------------------------------------------
no_key_press
    CALL KM_READ_KEY
    JR   C,no_key_press
    RET

; ---------------------------------------------------------------------------
; Print a string finished in $FF, using the firmware
; ---------------------------------------------------------------------------
print_menu
    LD   HL,menu_text
    LD   A,(HL)
.loop_print_text
    CALL TXT_OUTPUT
    INC  HL
    LD   A,(HL)
.continue_print_text
    CP   $FF
    JR   NZ,.loop_print_text
    RET

menu_text
    DEFB $1C,$00,$00,$00    ; INK 0,0,0     (Negro)
    DEFB $1C,$01,$00,$00    ; INK 1,0,0     (Negro)
    DEFB $1C,$02,$00,$00    ; INK 2,0,0     (Negro)
    DEFB $1C,$03,$00,$00    ; INK 3,0,0     (Negro)
    DEFB $1D,$00,$00        ; BORDER 0,0
    DEFB $04,$01            ; MODE 1
    DEFB $1F,1,10           ; LOCATE x,y 40 - 22 / 2
    DEFB $0F,1              ; Pen 3
    DEFB "- SFX Player (Press keys to hear SFXs) -"
    DEFB $1C,$01,$1A,$1A    ; INK 1,26,26   (Blanco)
    DEFB $FF 

; ---------------------------------------------------------------------------
; Insert the SFX bank
sfx_bank
    include "sfx_bank.asm"

; ---------------------------------------------------------------------------
    END
